//                                        First Created Date: Jan 20, 1998
//                                        Last Modified Date: Jan 22, 1998
//
//
// by Chih-Wei Huang             
//    cwhuang@trinova.com.tw
//
// Send comments or suggestions to me if you wish.
//
//
// Usage:
//
//  makedb [-o outputfile] [-s] [file1 file2 file3...]
//
//         -o:  specify the output file   Default: /etc/mail/deny
//         -s:  restart sendmail
//
//    If no source files specified, makedb automatically reads
//    files list in /etc/mail/deny.conf
//
//
// Compile:
//
//  You need gcc 2.7.x to compile it, since I use STL...:)
//
//  gcc -lstdc++ -ffor-scope makedb.cc -o makedb
//

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <fstream.h>
#include <string>
#include <set>


#define DBCONF    "/etc/mail/deny.conf"
#define DBFILE    "/etc/mail/deny"

const int buf_len=4096;

void appenddb(ostream& ofile, const string& ifname, char *msg)
{
	cout << "Appending " << ifname << "...\n";
	ifstream ifile(ifname.data());
	char buf[buf_len];
	while (ifile) {
		ifile.getline(buf, buf_len);
		int l=strlen(buf);
		if (l != 0) {
			if (buf[l-1] == '\r')
				buf[l-1] = '\0';
			ofile << buf << '\t' << msg << '\n';
		}
	}
	ifile.close();
}

int main(int argc, char **argv)
{
	bool restart_sendmail=false;
	const char *dbfile_name=DBFILE;
	char buf[buf_len];

	set<string, less<string> > sourcefiles;

	for (int i=1; i<argc; i++) {
		if (strcmp(argv[i], "-o") == 0)
			dbfile_name=argv[++i];
		else if (strcmp(argv[i], "-s") == 0)
			restart_sendmail=true;
		else
			sourcefiles.insert(argv[i]);
	}
	if (sourcefiles.size() == 0) {
		ifstream confile(DBCONF);
		while (confile) {
			confile.getline(buf, buf_len);
			if (buf[0] == '\0')
				break;
			else if (buf[0] != '#')
				sourcefiles.insert(buf);
		}
		confile.close();
	}	

	// Create the database text file
	cout << "Reading database files...\n";
	ofstream dbfile(dbfile_name);
	set<string, less<string> >::iterator it;
	for (it=sourcefiles.begin(); it != sourcefiles.end(); it++)
		appenddb(dbfile, *it, "SPAMMER");
	dbfile.close();

	// Create the database hash file
	cout << "Creating hash database files...\n";
	sprintf(buf, "makemap hash %s < %s", dbfile_name, dbfile_name);
	system(buf);
	
	// Restart sendmail daemon
	if (restart_sendmail) {
		pid_t sendmail_pid;
		ifstream pid_file("/var/run/sendmail.pid");
		pid_file >> sendmail_pid;
		kill(sendmail_pid, SIGHUP);
		cout << "Restarting sendmail daemon(" << sendmail_pid << ")...\n";
	}
	return 0;
}
